/*
 * C
 *
 * Copyright 2013-2015 IS2T. All rights reserved.
 * For demonstration purpose only.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

#include "LLCOMM_UART.h"
#define LLCOMM_BUFFERED_CONNECTION_IMPL LLCOMM_UART
#include "LLCOMM_BUFFERED_CONNECTION_impl.h"
#include "microej.h"
#include <stdio.h>
#include "stdlib.h"

#define COM_PORT_COUNT (2)

static LLCOMM_BUFFERED_CONNECTION* connections[COM_PORT_COUNT];

// data used for interrupt simulation only
static uint8_t rx_enabled[COM_PORT_COUNT];
static uint8_t tx_enabled[COM_PORT_COUNT];
static int32_t rx_data[3] = {'A', 'B', 'X'};
static int32_t rx_data_index = 0;

void comm_impl_event(int32_t code) {
	if (code != 0) return;
	if (tx_enabled[0]) {
		printf("Simulating TX interrupt for port 0\n");
		LLCOMM_BUFFERED_CONNECTION_transmitBufferReady(connections[0]);
	} else if (rx_enabled[0]) {
		printf("Simulating RX interrupt for port 0\n");
		LLCOMM_BUFFERED_CONNECTION_dataReceived(connections[0], rx_data[rx_data_index++]);
	}
	if (tx_enabled[1]) {
		printf("Simulating TX interrupt for port 1\n");
		LLCOMM_BUFFERED_CONNECTION_transmitBufferReady(connections[1]);
	} else if (rx_enabled[1]) {
		printf("Simulating RX interrupt for port 1\n");
		LLCOMM_BUFFERED_CONNECTION_dataReceived(connections[1], rx_data[rx_data_index++]);
	}
}

void LLCOMM_BUFFERED_CONNECTION_IMPL_initialize(LLCOMM_BUFFERED_CONNECTION* env)
{
	printf("LLCOMM_BUFFERED_CONNECTION_IMPL_initialize\n");
	if (((LLCOMM_UART*)env)->comIndex >= COM_PORT_COUNT) {
		printf("***** Port index greater than %i\n", COM_PORT_COUNT - 1);
		return;
	}
	// store connection to return it in the interrupt
	connections[((LLCOMM_UART*)env)->comIndex] = env;
}

int32_t LLCOMM_BUFFERED_CONNECTION_IMPL_getPlatformId(LLCOMM_BUFFERED_CONNECTION* env)
{
	printf("LLCOMM_BUFFERED_CONNECTION_IMPL_getPlatformId returns %i\n",
										((LLCOMM_UART*)env)->platformId);
	return ((LLCOMM_UART*)env)->platformId;
}

void* LLCOMM_BUFFERED_CONNECTION_IMPL_getName(LLCOMM_BUFFERED_CONNECTION* env)
{
	printf("LLCOMM_BUFFERED_CONNECTION_IMPL_getName\n");
	// this kind of connection cannot be open from a name (only from a platform port number)
	return 0;
}

int32_t LLCOMM_BUFFERED_CONNECTION_IMPL_configureDevice(LLCOMM_BUFFERED_CONNECTION* env, int32_t baudrate, int32_t bitsperchar, int32_t stopbits, int32_t parity)
{
	printf("LLCOMM_BUFFERED_CONNECTION_IMPL_configureDevice\n");
	return LLCOMM_BUFFERED_CONNECTION_SUCCESS;
}

void LLCOMM_BUFFERED_CONNECTION_IMPL_enableRX(LLCOMM_BUFFERED_CONNECTION* env)
{
	printf("LLCOMM_BUFFERED_CONNECTION_IMPL_enableRX\n");
}

void LLCOMM_BUFFERED_CONNECTION_IMPL_disableRX(LLCOMM_BUFFERED_CONNECTION* env)
{
	printf("LLCOMM_BUFFERED_CONNECTION_IMPL_disableRX\n");
}

void LLCOMM_BUFFERED_CONNECTION_IMPL_enableRXDeviceInterrupt(LLCOMM_BUFFERED_CONNECTION* env)
{
	printf("LLCOMM_BUFFERED_CONNECTION_IMPL_enableRXDeviceInterrupt\n");
	rx_enabled[((LLCOMM_UART*)env)->comIndex] = MICROEJ_TRUE;
}

void LLCOMM_BUFFERED_CONNECTION_IMPL_disableRXDeviceInterrupt(LLCOMM_BUFFERED_CONNECTION* env)
{
	printf("LLCOMM_BUFFERED_CONNECTION_IMPL_disableRXDeviceInterrupt\n");
	rx_enabled[((LLCOMM_UART*)env)->comIndex] = MICROEJ_FALSE;
}

void LLCOMM_BUFFERED_CONNECTION_IMPL_enableTX(LLCOMM_BUFFERED_CONNECTION* env)
{
	printf("LLCOMM_BUFFERED_CONNECTION_IMPL_enableTX\n");
}

void LLCOMM_BUFFERED_CONNECTION_IMPL_disableTX(LLCOMM_BUFFERED_CONNECTION* env)
{
	printf("LLCOMM_BUFFERED_CONNECTION_IMPL_disableTX\n");
}

void LLCOMM_BUFFERED_CONNECTION_IMPL_enableTXDeviceInterrupt(LLCOMM_BUFFERED_CONNECTION* env)
{
	printf("LLCOMM_BUFFERED_CONNECTION_IMPL_enableTXDeviceInterrupt\n");
	tx_enabled[((LLCOMM_UART*)env)->comIndex] = MICROEJ_TRUE;
}

void LLCOMM_BUFFERED_CONNECTION_IMPL_disableTXDeviceInterrupt(LLCOMM_BUFFERED_CONNECTION* env)
{
	printf("LLCOMM_BUFFERED_CONNECTION_IMPL_disableTXDeviceInterrupt\n");
	tx_enabled[((LLCOMM_UART*)env)->comIndex] = MICROEJ_FALSE;
}

int32_t LLCOMM_BUFFERED_CONNECTION_IMPL_getRXBufferStartAddress(LLCOMM_BUFFERED_CONNECTION* env)
{
	printf("LLCOMM_BUFFERED_CONNECTION_IMPL_getRXBufferStartAddress\n");
	return (int32_t)(((LLCOMM_UART*)env)->rxBuffer);
}

int32_t LLCOMM_BUFFERED_CONNECTION_IMPL_getRXBufferLength(LLCOMM_BUFFERED_CONNECTION* env)
{
	printf("LLCOMM_BUFFERED_CONNECTION_IMPL_getRXBufferLength\n");
	return LLCOMM_UART_RX_BUFFER_SIZE;
}

int32_t LLCOMM_BUFFERED_CONNECTION_IMPL_getTXBufferStartAddress(LLCOMM_BUFFERED_CONNECTION* env)
{
	printf("LLCOMM_BUFFERED_CONNECTION_IMPL_getTXBufferStartAddress\n");
	return (int32_t)(((LLCOMM_UART*)env)->txBuffer);
}

int32_t LLCOMM_BUFFERED_CONNECTION_IMPL_getTXBufferLength(LLCOMM_BUFFERED_CONNECTION* env)
{
	printf("LLCOMM_BUFFERED_CONNECTION_IMPL_getTXBufferLength\n");
	return LLCOMM_UART_TX_BUFFER_SIZE;
}

int32_t LLCOMM_BUFFERED_CONNECTION_IMPL_useTXHardwareFIFO(LLCOMM_BUFFERED_CONNECTION* env)
{
	printf("LLCOMM_BUFFERED_CONNECTION_IMPL_useTXHardwareFIFO\n");
	return LLCOMM_BUFFERED_CONNECTION_NO_HARDWARE_FIFO;
}

int32_t LLCOMM_BUFFERED_CONNECTION_IMPL_txFifoFull(LLCOMM_BUFFERED_CONNECTION* env)
{
	printf("LLCOMM_BUFFERED_CONNECTION_IMPL_txFifoFull\n");
	return LLCOMM_BUFFERED_CONNECTION_HARDWARE_FIFO_NOT_FULL;
}

void LLCOMM_BUFFERED_CONNECTION_IMPL_sendData(LLCOMM_BUFFERED_CONNECTION* env, int32_t data)
{
	printf("LLCOMM_BUFFERED_CONNECTION_IMPL_sendData: %i\n", data);
}

int32_t LLCOMM_BUFFERED_CONNECTION_IMPL_isTransmissionComplete(LLCOMM_BUFFERED_CONNECTION* env)
{
	printf("LLCOMM_BUFFERED_CONNECTION_IMPL_isTransmissionComplete\n");
	return LLCOMM_BUFFERED_CONNECTION_TRANSMISSION_COMPLETE;
}

int32_t LLCOMM_BUFFERED_CONNECTION_IMPL_getNbProperties(LLCOMM_BUFFERED_CONNECTION* env)
{
	printf("LLCOMM_BUFFERED_CONNECTION_IMPL_getNbProperties\n");
	// This implementation does not provide Device properties
	return 0;
}

void* LLCOMM_BUFFERED_CONNECTION_IMPL_getProperty(LLCOMM_BUFFERED_CONNECTION* env, void* propertyName)
{
	printf("LLCOMM_BUFFERED_CONNECTION_IMPL_getProperty\n");
	return NULL;
}

void* LLCOMM_BUFFERED_CONNECTION_IMPL_getPropertyName(LLCOMM_BUFFERED_CONNECTION* env, int32_t propertyId)
{
	printf("LLCOMM_BUFFERED_CONNECTION_IMPL_getPropertyName\n");
	return NULL;
}

void LLCOMM_BUFFERED_CONNECTION_IMPL_released(LLCOMM_BUFFERED_CONNECTION* env)
{
	printf("LLCOMM_BUFFERED_CONNECTION_IMPL_released\n");
	// This implementation does not support dynamic connections add/removal.
	return;
}
