/**
 * Java
 *
 * Copyright 2013 IS2T. All rights reserved.
 * For demonstration purpose only
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.is2t.examples.ecom.comm;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import ej.ecom.io.Connection;
import ej.ecom.io.Connector;
import ej.ecom.io.InputConnection;
import ej.ecom.io.OutputConnection;

public class EcomCommExample {
	
	/**
	 * Comm connection string: used to open the comm connection
	 */
	private static final String CONNECTION_STRING = "comm:com1";
	
	/**
	 * String to output through the comm connection
	 */
	private static final String OUTPUT_STRING = "123";

	private Connection connection;	// current comm connection
	private OutputStream out;		// current comm connection's output stream
	private InputStream in;		    // current comm connection's input stream

	public static void main(String[] args) {
		EcomCommExample example = new EcomCommExample();
		try {
			example.launch(CONNECTION_STRING);
		} catch (IOException e) {
			e.printStackTrace();
			return;
		}
	}
	
	private void launch(String connectionString) throws IOException {
		connection = openConnection(connectionString);
		
		out = openOutputStream(connection);
		
		out.write(OUTPUT_STRING.getBytes());
		showMessage("Written data");
		out.flush();
		showMessage("Flushed data");

		closeOutputStream(out);

		in = openInputStream(connection);
	
		int input;
		do {
			input = in.read();
			showMessage("Read data: " + new String(new byte[] {(byte) input}));
		} while (input != 'X');

		closeInputStream(in);

		closeConnection(connection);
	}

	private Connection openConnection(String connectionString) throws IOException {
		Connection comm = Connector.open(connectionString);
		showMessage("Comm connection opened");
		return comm;
	}

	private void closeConnection(Connection connection) throws IOException {
		connection.close();
		showMessage("Comm connection closed");
	}

	private InputStream openInputStream(Connection connection) throws IOException {
		InputStream is = ((InputConnection)connection).openInputStream();
		showMessage("Input stream opened");
		return is;
	}

	private void closeInputStream(InputStream in) throws IOException {
		in.close();
		showMessage("Input stream closed.");
	}

	private OutputStream openOutputStream(Connection connection) throws IOException {
		OutputStream os = ((OutputConnection)connection).openOutputStream();
		showMessage("Output stream opened");
		return os;
	}

	private void closeOutputStream(OutputStream out) throws IOException {
		out.close();
		showMessage("Output stream closed");
	}

	private void showMessage(String message) {
		StringBuffer sb = new StringBuffer();
		sb.append("[LOG] ");
		if (message != null)
			sb.append(message);
		System.out.println(sb.toString());
	}
}
